package database

// GetLastUsedPic
func (db *Database) GetLastUsedPic(increment bool) (int64, string, error) {
	min := db.getPicMinViewCount()
	var id int64
	var fileName string
	row, err := db.db.Query("SELECT id, filename FROM pics WHERE viewed = ? ORDER BY RANDOM() ASC LIMIT 1", min)
	if err != nil {
		return 0, "", err
	}

	defer row.Close()
	for row.Next() {
		row.Scan(&id, &fileName)
	}

	if increment {
		// Increment the view count
		db.incrementPicViewCount(id)
	}

	return id, fileName, nil
}

// DeleteAllpics
func (db *Database) DeleteAllPics() {
	db.db.Exec("DELETE FROM pics")
}

// StorePic
func (db *Database) StorePic(filename string) (int64, error) {
	min := db.getPicMinViewCount()
	exec, err := db.db.Exec(
		"INSERT INTO `pics` (`filename`, `viewed`) VALUES (?,?)",
		filename,
		min,
	)

	if err != nil {
		return 0, err
	}

	return exec.LastInsertId()
}

// DeletePic
func (db *Database) DeletePic(id int64) (int64, error) {
	res, _ := db.db.Exec("DELETE FROM `pics` WHERE `id` = ?", id)
	return res.RowsAffected()
}

// DeletePic
func (db *Database) ResetPics() {
	db.db.Exec("UPDATE `pics` set `viewed` = 0")
}

// GetAllPics
func (db *Database) GetAllPics() (map[int64]string, error) {
	pics := make(map[int64]string)
	row, err := db.db.Query("SELECT id, filename FROM pics")
	if err != nil {
		return pics, err
	}

	defer row.Close()
	for row.Next() {
		var pic string
		var id int64
		row.Scan(&id, &pic)
		pics[id] = pic
	}

	return pics, nil
}

func (db *Database) getPicMinViewCount() int64 {
	var count int64
	db.db.QueryRow("SELECT MIN(`viewed`) FROM `pics`").Scan(&count)
	return count
}

func (db *Database) incrementPicViewCount(id int64) error {
	_, err := db.db.Exec("UPDATE `pics` SET `viewed` = `viewed` + 1 WHERE `id` = ?", id)
	return err
}
