package gomatrixbot

import (
	"context"
	"fmt"
	"image"
	"os"
	"strings"
	"time"

	"github.com/fogleman/gg"
	"github.com/nfnt/resize"
	"maunium.net/go/mautrix/event"
	"maunium.net/go/mautrix/id"
)

const fontSize = 42

const BIRB_PATH = BASE_PATH + "/reaction_pics"

func (mtrx *MtrxClient) postBirb(ctx context.Context, evt *event.Event) {
	idd, birb, err := mtrx.db.GetLastUsedPic(true)
	if err != nil {
		mtrx.c.Log.Err(err).Msg("Failed to get birb")
		return
	}

	b, err := os.ReadFile(BIRB_PATH + "/" + birb)
	if err != nil {
		mtrx.c.Log.Err(err).Msg("Failed to read new file")
		mtrx.db.DeletePic(idd)
		mtrx.sendMessage(ctx, evt.RoomID, "Failed to read file: "+birb)
		return
	}

	fileParts := strings.Split(birb, ".")
	fileType := fileParts[len(fileParts)-1]
	mime := "image/png"
	if fileType == "jpg" || fileType == "jpeg" {
		mime = "image/jpeg"
	}

	media, err := mtrx.c.UploadBytesWithName(ctx, b, mime, birb)
	if err != nil {
		mtrx.c.Log.Err(err).Msg("Failed to upload file")
		return
	}

	_, _ = mtrx.c.SendMessageEvent(ctx, evt.RoomID, event.EventMessage, &event.MessageEventContent{
		MsgType:  event.MsgImage,
		FileName: birb,
		Body:     "",
		Info: &event.FileInfo{
			MimeType: "image/png",
		},
		URL:      id.ContentURIString(media.ContentURI.String()),
		Mentions: &event.Mentions{},
	})
}

func (mtrx *MtrxClient) postBirbWithText(ctx context.Context, evt *event.Event, text string, filename string) {
	increment := true
	if filename != "" {
		increment = false
	}

	// Do a lookup
	dbId, birb, err := mtrx.db.GetLastUsedPic(increment)
	if err != nil {
		mtrx.sendMessage(ctx, evt.RoomID, "Failed to get an image "+err.Error()+". "+birb)
		return
	}
	reader, err := os.Open(BIRB_PATH + "/" + birb)
	if err != nil {
		mtrx.sendMessage(ctx, evt.RoomID, "Failed to load image "+err.Error()+". "+birb)
		if filename != "" {
			for k, v := range mtrx.pics {
				if v == birb {
					dbId = k
					break
				}
			}
		}
		mtrx.db.DeletePic(dbId)
		delete(mtrx.pics, dbId)
		return
	}

	// do the swaps
	img, _, err := image.Decode(reader)
	if err != nil {
		mtrx.sendMessage(ctx, evt.RoomID, "I can't read this image. I give up. "+err.Error()+". "+birb)
		return
	}

	imgNew := resize.Resize(1024, 1024, img, resize.Lanczos2)
	m := gg.NewContextForImage(imgNew)

	w := 1024
	h := 1024

	// m.LoadFontFace("/usr/share/fonts/truetype/msttcorefonts/Comic_Sans_MS.ttf", fontSize)
	m.LoadFontFace("/root/gomatrixbot/OneNightSansBlack.ttf", fontSize)

	m.SetHexColor("#000")
	text = strings.ReplaceAll(text, "\n", " ")

	// Apply white fill
	m.SetHexColor("#FFF")

	chonks := strings.Split(text, " ")
	chinks := []string{}
	tmp := ""
	for _, v := range chonks {
		tmp = tmp + " " + v
		if len(tmp) > 38 {
			chinks = append(chinks, tmp)
			tmp = ""
		}
	}

	if tmp != "" {
		chinks = append(chinks, tmp)
	}

	strokeSize := 6
	ayyyy := float64(h) - fontSize
	for i := len(chinks) - 1; i >= 0; i-- {
		m.SetHexColor("#000")
		for dy := -strokeSize; dy <= strokeSize; dy++ {
			for dx := -strokeSize; dx <= strokeSize; dx++ {
				// give it rounded corners
				if dx*dx+dy*dy >= strokeSize*strokeSize {
					continue
				}
				x := float64(w/2 + dx)
				y := ayyyy + float64(dy)
				m.DrawStringAnchored(chinks[i], x, y, 0.5, 0.5)
			}
		}

		m.SetHexColor("#FFF")
		m.DrawStringAnchored(chinks[i], float64(w)/2, ayyyy, 0.5, 0.5)
		ayyyy = ayyyy - (fontSize + 10)
	}

	rand := time.Now().UnixNano()
	newLocalFile := fmt.Sprintf("%s/new_%d.png", OUTPUT_PATH, rand)
	m.SavePNG(newLocalFile)
	b, err := os.ReadFile(newLocalFile)
	if err != nil {
		mtrx.c.Log.Err(err).Msg("Failed to read new file")
		return
	}

	// upload
	media, err := mtrx.c.UploadBytesWithName(ctx, b, "image/png", birb)
	if err != nil {
		mtrx.c.Log.Err(err).Msg("Failed to upload file")
		return
	}

	// get
	resp, err := mtrx.c.SendMessageEvent(ctx, evt.RoomID, event.EventMessage, &event.MessageEventContent{
		MsgType:  event.MsgImage,
		FileName: birb,
		Info: &event.FileInfo{
			MimeType: "image/png",
		},
		Body: "",
		URL:  id.ContentURIString(media.ContentURI.String()),
	})

	mtrx.savedFiles[resp.EventID] = newLocalFile
}

// UpdatePics
func (mtrx *MtrxClient) UpdatePics() error {
	files, err := os.ReadDir(BIRB_PATH)
	if err != nil {
		return err
	}

	for _, file := range files {
		found := false

		if file.IsDir() || strings.HasPrefix(file.Name(), ".") {
			continue
		}

		// check if in our map
		for _, v := range mtrx.pics {
			if v == file.Name() {
				found = true
				break
			}
		}

		if found {
			continue
		}

		idd, err := mtrx.db.StorePic(file.Name())
		if err != nil {
			return err
		}

		mtrx.sendMessage(context.Background(), mtrx.adminRoom, "New pic added: "+file.Name())
		mtrx.pics[idd] = file.Name()
	}

	return nil
}

func ChunkString(s string, chunkSize int) []string {
	var chunks []string
	runes := []rune(s)

	if len(runes) == 0 {
		return []string{s}
	}

	for i := 0; i < len(runes); i += chunkSize {
		nn := i + chunkSize
		if nn > len(runes) {
			nn = len(runes)
		}
		chunks = append(chunks, string(runes[i:nn]))
	}
	return chunks
}
