package gomatrixbot

import (
	"context"
	"time"

	"maunium.net/go/mautrix/id"
)

func (mtrx *MtrxClient) getCounterFor(ctx context.Context, roomID id.RoomID, key string) {
	counter, err := mtrx.db.GetCounterFor(key)
	if err != nil {
		mtrx.sendMessage(ctx, roomID, "Failed to get counter for "+key+": "+err.Error())
		return
	}

	diff := time.Now().Sub(counter)
	msg := "Days since " + key + ": " + diff.String()

	mtrx.sendMessage(ctx, roomID, msg)
}

func (mtrx *MtrxClient) getAllCounters(ctx context.Context, roomID id.RoomID) {
	counters, err := mtrx.db.GetAllCounters()
	if err != nil {
		mtrx.sendMessage(ctx, roomID, "Failed to get counters: "+err.Error())
		return
	}

	msg := "Counters:\n"
	for key, timestamp := range counters {
		msg += key + ": " + timestamp.String() + "\n"
	}

	mtrx.sendMessage(ctx, roomID, msg)
}

func (mtrx *MtrxClient) updateCounterFor(ctx context.Context, roomID id.RoomID, key string) {
	timestamp := time.Now()
	_, err := mtrx.db.UpdateCounterFor(key, timestamp)
	if err != nil {
		mtrx.sendMessage(ctx, roomID, "Failed to update counter for "+key+": "+err.Error())
		return
	}

	mtrx.sendMessage(ctx, roomID, "Updated counter for "+key)
}

func (mtrx *MtrxClient) deleteCounterFor(ctx context.Context, roomID id.RoomID, key string) {
	_, err := mtrx.db.DeleteCounter(key)
	if err != nil {
		mtrx.sendMessage(ctx, roomID, "Failed to delete counter for "+key+": "+err.Error())
		return
	}

	mtrx.sendMessage(ctx, roomID, "Deleted counter for "+key)
}
