package gomatrixbot

import (
	"context"
	"fmt"
	"math/rand"
	"os"
	"os/exec"
	"path/filepath"
	"strings"
	"time"

	"maunium.net/go/mautrix/id"
)

const (
	faceSwapSrc = "/root/faceswap"

	faceSwapCmd = "facefusion.py headless-run " +
		"--execution-thread-count 8 " +
		"--system-memory-limit 4 " +
		"--face-selector-mode many " +
		"-s %s -t %s -o %s"
)

// swapFace swaps the face of the user with the face of the person in the image.
func (mtrx *MtrxClient) swapFace(ctx context.Context, channel id.RoomID, fileToSwap string) {
	files, _ := os.ReadDir(faceSwapSrc)
	fileList := make([]string, 0)
	for _, file := range files {
		if file.IsDir() || strings.HasPrefix(file.Name(), ".") {
			continue
		}

		fileList = append(fileList, file.Name())
	}

	// Pick a src file
	faceFile := fmt.Sprintf(faceSwapSrc + "/" + fileList[rand.Intn(len(fileList))])

	if !strings.HasSuffix(faceFile, ".jpg") {
		cmd := exec.Command("/usr/bin/convert", faceFile, faceFile+".jpg")
		err := cmd.Run()
		if err != nil {
			mtrx.sendMessage(ctx, channel, "Error converting: "+faceFile+":"+err.Error())
			return
		}

		faceFile = faceFile + ".jpg"
		defer os.Remove(faceFile)
	}

	if !strings.HasSuffix(fileToSwap, ".jpg") {
		cmd := exec.Command("/usr/bin/convert", fileToSwap, fileToSwap+".jpg")
		err := cmd.Run()
		if err != nil {
			mtrx.sendMessage(ctx, channel, "Error converting: "+fileToSwap+":"+err.Error())
			return
		}

		fileToSwap = fileToSwap + ".jpg"
		defer os.Remove(fileToSwap)
	}

	swapFile := fmt.Sprintf(OUTPUT_PATH + "/" + filepath.Base(fileToSwap))
	tempFile := fmt.Sprintf("%s/fs_%d.jpg", OUTPUT_PATH, time.Now().UnixNano())

	shellCmd := fmt.Sprintf(faceSwapCmd, faceFile, swapFile, tempFile)

	cmd := exec.Command("/usr/bin/python3", strings.Split(shellCmd, " ")...)
	cmd.Dir = "/root/facefusion"
	err := cmd.Run()
	if err != nil {
		mtrx.sendMessage(ctx, channel, "Failed to run face swap: "+err.Error())
		return
	}

	// Put dst file
	mtrx.uploadAndPostFile(ctx, channel, tempFile)

	// Remove tmp file
	os.Remove(tempFile)
}
