package gomatrixbot

import (
	"context"
	"fmt"
	"net/http"
	"time"

	"github.com/StalkR/imdb"
	"maunium.net/go/mautrix/event"
	"maunium.net/go/mautrix/id"
)

// IMDb deployed awswaf and denies requests using the default Go user-agent (Go-http-client/1.1).
// For now it still allows requests from a browser user-agent. Remain respectful, no spam, etc.
const userAgent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36"

type customTransport struct {
	http.RoundTripper
}

func (e *customTransport) RoundTrip(r *http.Request) (*http.Response, error) {
	defer time.Sleep(time.Second)         // don't go too fast or risk being blocked by awswaf
	r.Header.Set("Accept-Language", "en") // avoid IP-based language detection
	r.Header.Set("User-Agent", userAgent)
	return e.RoundTripper.RoundTrip(r)
}

func (mtrx *MtrxClient) searchImdb(ctx context.Context, roomID id.RoomID, query string) {
	client := &http.Client{
		Transport: &customTransport{http.DefaultTransport},
	}
	title, err := imdb.SearchTitle(client, query)
	if err != nil {
		mtrx.sendMessage(ctx, roomID, "Error searching: "+err.Error())
		return
	}
	if len(title) == 0 {
		mtrx.sendMessage(ctx, roomID, "Not found")
		return
	}

	t, err := imdb.NewTitle(client, title[0].ID)
	if err != nil {
		mtrx.sendMessage(ctx, roomID, "Error loading:"+err.Error())
		return
	}

	content, err := mtrx.getFileContent(t.Poster.ContentURL)
	if err == nil {
		// Upload
		media, err := mtrx.c.UploadBytesWithName(ctx, content, "image/jpg", "poster.jpg")
		if err != nil {
			mtrx.c.Log.Err(err).Msg("Failed to upload file")
			return
		}
		_, _ = mtrx.c.SendMessageEvent(ctx, roomID, event.EventMessage, &event.MessageEventContent{
			MsgType:  event.MsgImage,
			Body:     "",
			FileName: "poster.jpg",
			Info: &event.FileInfo{
				MimeType: "image/jpg",
			},
			URL: id.ContentURIString(media.ContentURI.String()),
		})
	}

	msg := fmt.Sprintf(
		"%s - %s (%d)\nRating: %s (%d votes)\nDuration: %s\nGenres: %s\nLanguages: %s\nDescription: %s",
		t.Name,
		t.Type,
		t.Year,
		t.Rating,
		t.RatingCount,
		t.Duration,
		t.Genres,
		t.Languages,
		t.Description,
	)

	// ID            string   `json:",omitempty"`
	// URL           string   `json:",omitempty"`
	// Name          string   `json:",omitempty"`
	// Type          string   `json:",omitempty"`
	// Year          int      `json:",omitempty"`
	// Rating        string   `json:",omitempty"`
	// RatingCount   int      `json:",omitempty"`
	// Duration      string   `json:",omitempty"`
	// Directors     []Name   `json:",omitempty"`
	// Writers       []Name   `json:",omitempty"`
	// Actors        []Name   `json:",omitempty"`
	// Genres        []string `json:",omitempty"`
	// Languages     []string `json:",omitempty"`
	// Nationalities []string `json:",omitempty"`
	// Description   string   `json:",omitempty"`
	// Poster        Media    `json:",omitempty"`

	mtrx.sendMessage(ctx, roomID, msg)
}
