package gomatrixbot

import (
	"context"
	"encoding/json"
	"fmt"
	"io"
	"net/http"
	"strings"
	"time"

	"maunium.net/go/mautrix/id"
)

type metarResp []struct {
	RawOb  string `json:"rawOb"`
	RawTaf string `json:"rawTaf"`
}

func (mtrx *MtrxClient) sendMetar(ctx context.Context, roomID id.RoomID, station string) {
	url := "https://aviationweather.gov/api/data/metar"
	httpClient := http.Client{Timeout: time.Second * 5}
	req, err := http.NewRequest(http.MethodGet, url, nil)
	if err != nil {
		mtrx.sendMessage(ctx, roomID, "Error fetching METAR: "+err.Error())
		return
	}

	q := req.URL.Query()
	q.Add("ids", strings.ToUpper(station))
	q.Add("format", "json")
	q.Add("taf", "true")
	req.URL.RawQuery = q.Encode()
	res, getErr := httpClient.Do(req)
	if getErr != nil {
		mtrx.sendMessage(ctx, roomID, "Error fetching METAR feed: "+getErr.Error())
		return
	}

	if res.Body != nil {
		defer res.Body.Close()
	}

	body, readErr := io.ReadAll(res.Body)
	if readErr != nil {
		mtrx.sendMessage(ctx, roomID, "Error reading METAR feed: "+readErr.Error())
		return
	}

	// Decode into weatherResponse struct
	metar := metarResp{}
	jsonErr := json.Unmarshal(body, &metar)
	if jsonErr != nil {
		mtrx.sendMessage(ctx, roomID, "Error decoding METAR: "+jsonErr.Error())
		return
	}

	if len(metar) == 0 {
		mtrx.sendMessage(ctx, roomID, "No METAR data found for "+station)
		return
	}
	mtrx.sendMessage(ctx, roomID, fmt.Sprintf("%s", metar[0].RawOb))
}

func (mtrx *MtrxClient) sendTaf(ctx context.Context, roomID id.RoomID, station string) {
	url := "https://aviationweather.gov/api/data/taf"
	httpClient := http.Client{Timeout: time.Second * 5}
	req, err := http.NewRequest(http.MethodGet, url, nil)
	if err != nil {
		mtrx.sendMessage(ctx, roomID, "Error fetching METAR: "+err.Error())
		return
	}

	q := req.URL.Query()
	q.Add("ids", strings.ToUpper(station))
	q.Add("format", "raw")
	q.Add("metar", "false")
	req.URL.RawQuery = q.Encode()
	res, getErr := httpClient.Do(req)
	if getErr != nil {
		mtrx.sendMessage(ctx, roomID, "Error fetching METAR feed: "+getErr.Error())
		return
	}

	if res.Body != nil {
		defer res.Body.Close()
	}

	body, readErr := io.ReadAll(res.Body)
	if readErr != nil {
		mtrx.sendMessage(ctx, roomID, "Error reading METAR feed: "+readErr.Error())
		return
	}

	mtrx.sendMessage(ctx, roomID, string(body))
}
