package gomatrixbot

import (
	"encoding/json"
	"fmt"
	"io"
	"net/http"
	"net/url"
	"time"
)

const UrbanDictionaryApi = "https://api.urbandictionary.com/v0/define?term="

type UrbanDictionaryResponse struct {
	List []struct {
		Definition  string    `json:"definition"`
		Permalink   string    `json:"permalink"`
		ThumbsUp    int       `json:"thumbs_up"`
		Author      string    `json:"author"`
		Word        string    `json:"word"`
		Defid       int       `json:"defid"`
		CurrentVote string    `json:"current_vote"`
		WrittenOn   time.Time `json:"written_on"`
		Example     string    `json:"example"`
		ThumbsDown  int       `json:"thumbs_down"`
	} `json:"list"`
}

func (mtrx *MtrxClient) getUrbanDictionary(search string) string {
	res, err := http.Get(UrbanDictionaryApi + url.QueryEscape(search))
	if err != nil {
		return err.Error()
	}

	resBody, err := io.ReadAll(res.Body)
	if err != nil {
		return err.Error()
	}

	var ud UrbanDictionaryResponse
	err = json.Unmarshal(resBody, &ud)

	if len(ud.List) == 0 {
		return "No results found."
	}

	return fmt.Sprintf(
		"Definition of %s:\n%s\n\nExample:\n%s",
		ud.List[0].Word,
		ud.List[0].Definition,
		ud.List[0].Example,
	)
}
