package gomatrixbot

import (
	"bytes"
	"context"
	"encoding/json"
	"io"
	"net/http"

	"maunium.net/go/mautrix/event"
	"maunium.net/go/mautrix/id"
)

// This is free - you can get your own. Or don't _/\o/\_
var xkcdSearchUrl = "https://qtg5aekc2iosjh93p.a1.typesense.net/multi_search?use_cache=true&x-typesense-api-key=8hLCPSQTYcBuK29zY5q6Xhin7ONxHy99"

type XKCDSearchRequest struct {
	Searches []XKCDSearch `json:"searches"`
}

type XKCDSearch struct {
	QueryBy             string `json:"query_by"`
	QueryByWeights      string `json:"query_by_weights"`
	NumTypos            int    `json:"num_typos"`
	ExcludeFields       string `json:"exclude_fields"`
	VectorQuery         string `json:"vector_query"`
	HighlightFullFields string `json:"highlight_full_fields"`
	Collection          string `json:"collection"`
	Q                   string `json:"q"`
	FacetBy             string `json:"facet_by"`
	MaxFacetValues      int    `json:"max_facet_values"`
	Page                int    `json:"page"`
	PerPage             int    `json:"per_page"`
}

type XKCDSearchResult struct {
	Results []struct {
		Hits []struct {
			Document struct {
				AltTitle   string `json:"altTitle"`
				ID         string `json:"id"`
				ImageURL   string `json:"imageUrl"`
				Title      string `json:"title"`
				Transcript string `json:"transcript"`
			} `json:"document"`
		} `json:"hits"`
	} `json:"results"`
}

func (mtrx *MtrxClient) searchXKCD(ctx context.Context, roomID id.RoomID, query string) {
	search := XKCDSearch{
		QueryBy:             "title,altTitle,transcript,topics,embedding",
		QueryByWeights:      "127,80,80,1,1",
		NumTypos:            1,
		ExcludeFields:       "embedding",
		VectorQuery:         "embedding:([], k: 30, distance_threshold: 0.1, alpha: 0.9)",
		HighlightFullFields: "title,altTitle,transcript,topics,embedding",
		Collection:          "xkcd",
		Q:                   query,
		FacetBy:             "topics,publishDateYear",
		MaxFacetValues:      100,
		Page:                1,
		PerPage:             5,
	}
	searchReq := XKCDSearchRequest{
		Searches: []XKCDSearch{search},
	}

	reqData, err := json.Marshal(searchReq)
	if err != nil {
		mtrx.sendMessage(ctx, roomID, err.Error())
		return
	}

	req, err := http.NewRequest("POST", xkcdSearchUrl, bytes.NewBuffer(reqData))
	if err != nil {
		mtrx.sendMessage(ctx, roomID, err.Error())
		return
	}

	req.Header.Set("content-type", "application/json")
	req.Header.Set("origin", "https://findxkcd.com")
	req.Header.Set("referrer", "https://findxkcd.com/")
	req.Header.Set("user-agent", "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36")

	client := &http.Client{}
	resp, err := client.Do(req)
	if err != nil {
		mtrx.sendMessage(ctx, roomID, err.Error())
		return
	}
	defer resp.Body.Close()

	body, _ := io.ReadAll(resp.Body)

	var searchResult XKCDSearchResult
	err = json.Unmarshal(body, &searchResult)
	if err != nil {
		mtrx.sendMessage(ctx, roomID, err.Error())
		return
	}

	if len(searchResult.Results) == 0 || len(searchResult.Results[0].Hits) == 0 {
		mtrx.sendMessage(ctx, roomID, "No results found")
		return
	}

	content, err := mtrx.getFileContent(searchResult.Results[0].Hits[0].Document.ImageURL)
	if err != nil {
		mtrx.sendMessage(ctx, roomID, err.Error())
		return
	}

	// Upload
	mtrx.sendMessage(ctx, roomID, searchResult.Results[0].Hits[0].Document.Title)
	media, err := mtrx.c.UploadBytesWithName(ctx, content, "image/png", "xkcd.png")
	if err != nil {
		mtrx.c.Log.Err(err).Msg("Failed to upload file")
		return
	}

	_, err = mtrx.c.SendMessageEvent(ctx, roomID, event.EventMessage, &event.MessageEventContent{
		MsgType:  event.MsgImage,
		Body:     "",
		URL:      id.ContentURIString(media.ContentURI.String()),
		FileName: "xkcd.png",
		Mentions: &event.Mentions{},
	})

	if err != nil {
		mtrx.sendMessage(ctx, roomID, err.Error())
	}

}
